using System;
using System.Globalization;
using System.Text;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms;
using Syncfusion.Drawing;

namespace InPlaceRichTextCellSample
{
	#region cell model class

	public class InPlaceRichTextCellModel : GridTextBoxCellModel
	{
		private RichTextBox rtb;

		public InPlaceRichTextCellModel(GridModel grid)
			: base(grid)
		{
			rtb = new RichTextBox();
			rtb.ScrollBars = RichTextBoxScrollBars.None;

			rtb.Font	= new System.Drawing.Font("Arial", 10F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));

			rtb.Location = new Point(-1000, -1000);
			rtb.Visible = false;
			rtb.Text = string.Empty;
		}


		private InPlaceRichTextCellRenderer renderer = null;
		/// <override/>
		public override GridCellRendererBase CreateRenderer(GridControlBase control)
		{
			renderer = new InPlaceRichTextCellRenderer(control, this);
			return renderer;
		}

		/// <override/>
		protected override Size OnQueryPrefferedClientSize(Graphics g, int rowIndex, int colIndex, GridStyleInfo style, GridQueryBounds queryBounds)
		{
			//don't size col width...
			//  'in versions >= 2.0.6.0, use WinFormsUtils.MeasureSampleWString instead of GridCellModelBase.MeasureSampleWString
			// Size clientSize = new Size(this.Grid.ColWidths[colIndex], WinFormsUtils.MeasureSampleWString(g, style.GdipFont).Height);
			Size clientSize = new Size(this.Grid.ColWidths[colIndex], GridCellModelBase.MeasureSampleWString(g, style.GdipFont).Height);
			
			rtb.Width = clientSize.Width - style.BorderMargins.Left - style.BorderMargins.Right - this.ButtonBarSize.Width;
			
			if (Grid.ActiveGridView == null || !Grid.ActiveGridView.CurrentCell.HasCurrentCellAt(rowIndex, colIndex)
				|| Grid.ActiveGridView.CurrentCell.StaticDrawing)
			{
				if(style.CellValue != null)
				{	
					rtb.Rtf = style.CellValue.ToString();
				}
				else
				{
					rtb.Rtf = @"{\rtf1\ansi\ansicpg1252\deff0\deflang1033\viewkind4\uc1\pard\f0\fs17\par}";
				}
			}
			else
				rtb.Rtf = ((RichTextBox)this.renderer.TextBox).Rtf;
			//insert a /par so we get top of next line
			rtb.AutoSize=true;
			rtb.SelectionLength = 0;//rtb.TextLength;
			rtb.SelectionLength = 0;
			rtb.SelectedRtf = @"{\rtf1\ansi\ansicpg1252\deff0\deflang1033\viewkind4\uc1\pard\f0\fs17\par}";
			
			int height =  rtb.GetPositionFromCharIndex(rtb.TextLength).Y  + style.BorderMargins.Top + style.BorderMargins.Bottom + rtb.SelectionFont.Height / 2;

			return new Size(this.Grid.ColWidths[colIndex], height);
		}
	}
	#endregion

	#region cell renderer class

	public class InPlaceRichTextCellRenderer : GridTextBoxCellRenderer 
	{
		//used for in place editing
		GridTextBoxControl activeRichTextBox;
		RichTextBox staticRichTextBox;

		//used to display context menu for active edit control
		//public so the user can modify it 
		public ContextMenu cmnuGridRichText;

		public InPlaceRichTextCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
			: base(grid, cellModel)
		{

			SupportsFocusControl = true;
			activeRichTextBox = this.TextBox as GridTextBoxControl;
			staticRichTextBox = new RichTextBox();

			staticRichTextBox.Font	= new System.Drawing.Font("Arial", 10F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));

			activeRichTextBox.Font	= new System.Drawing.Font("Arial", 10F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));

			activeRichTextBox.BorderStyle = BorderStyle.None;
			staticRichTextBox.BorderStyle = BorderStyle.None;
			
			cmnuGridRichText = new ContextMenu(new MenuItem[]
												{
													new MenuItem("Change &Font", new EventHandler(fontClick), System.Windows.Forms.Shortcut.CtrlF),
													new MenuItem("Cu&t", new EventHandler(cutClick),System.Windows.Forms.Shortcut.CtrlX),
													new MenuItem("&Copy", new EventHandler(copyClick),System.Windows.Forms.Shortcut.CtrlC),
													new MenuItem("&Paste", new EventHandler(pasteClick),System.Windows.Forms.Shortcut.CtrlV),
													new MenuItem("-"), 
												});

			activeRichTextBox.ContextMenu = cmnuGridRichText;
			cmnuGridRichText.Popup += new EventHandler(contextMenu_Popup);
		}

		#region Context menu handlers

		/// <summary>
		///		This method is used to disable control(Cut,Copy,Paste)Initinitially
		/// </summary>
		/// 
		/// <returns>
		///     A void value...
		/// </returns>
		private void contextMenu_Popup(object sender, EventArgs e)
		{
			cmnuGridRichText.MenuItems[2].Enabled = CanCut();
			cmnuGridRichText.MenuItems[3].Enabled = CanCopy();
			cmnuGridRichText.MenuItems[4].Enabled = CanPaste();
		}

		
		/// <summary>
		/// Copy
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void copyClick(object sender, EventArgs e)
		{
			Copy();
		}

		/// <summary>
		/// Paste
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void pasteClick(object sender, EventArgs e)
		{
			Paste();
		}

		/// <summary>
		///  Cut
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void cutClick(object sender, EventArgs e)
		{
			Cut();
		}
		
		/// <summary>
		///  Font Change
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void fontClick(object sender, EventArgs e)
		{
			ChangeFont();
		}

		/// <summary>
		///  Font Change
		/// </summary>
		private void ChangeFont()
		{
			FontDialog fontDialog1 = new FontDialog();
			fontDialog1.ShowColor = true;
			fontDialog1.Font = activeRichTextBox.SelectionFont;
			fontDialog1.Color = activeRichTextBox.SelectionColor;
			this.CurrentCell.ConfirmChanges();

			int oldHeight=activeRichTextBox.Height;

			if(fontDialog1.ShowDialog() != DialogResult.Cancel )
			{
				activeRichTextBox.Focus();
				activeRichTextBox.SelectionFont = fontDialog1.Font;
				activeRichTextBox.SelectionColor = fontDialog1.Color;
				SetHeight();
			}
		}

		/// <summary>
		/// Set's the Row Height
		/// </summary>
		private void SetHeight()
		{
			int height = activeRichTextBox.GetPositionFromCharIndex(activeRichTextBox.TextLength).Y +
				activeRichTextBox.SelectionFont.Height / 2;

			int startIndex = activeRichTextBox.SelectionStart;
			int endIndex = startIndex + activeRichTextBox.SelectedText.Length -1;
			int lastLine = activeRichTextBox.GetLineFromCharIndex(activeRichTextBox.TextLength);
			int sellastLine =  activeRichTextBox.GetLineFromCharIndex(endIndex);

			if(lastLine == sellastLine)
				height +=  activeRichTextBox.SelectionFont.Height;

			this.Grid.Model.RowHeights[RowIndex] = height;
			this.CurrentCell.ConfirmChanges();
		}

		#endregion

		#region helper methods for the renderer class

		//used to set cell changes flag
		private void activeRichTextBox_TextChanged(object sender, EventArgs e)
		{
			this.Grid.CurrentCell.IsModified = true;
		}

		void AssignRtf(RichTextBox rtb, string rtf)
		{
			if (Syncfusion.Drawing.RichTextPaint.IsValidRtf(rtf))
				rtb.Rtf = rtf;
			else 
				rtb.Text = rtf;
		}

		protected void FixControlParent(Control control)
		{
			if (control.Parent != Grid || !control.Visible)
			{
				control.Location = new Point(10000, 10000);

				control.CausesValidation = false;
				control.Anchor = AnchorStyles.None;
				control.Dock = DockStyle.None;
				Grid.GetWindow().Controls.Add(control);
				control.Visible = true;
				SetControl(control);
			}
		}

		#endregion

		#region base class overrides

		/// <override/>
		protected override void OnInitialize(int rowIndex, int colIndex)
		{
			base.OnInitialize(rowIndex, colIndex);
			ControlValue = this.Grid.Model[rowIndex, colIndex].CellValue;
			FixControlParent(activeRichTextBox);
			if(this.ControlValue == null)
				this.ControlValue = "";
			AssignRtf(activeRichTextBox, this.ControlValue.ToString());
			
			this.Grid.CurrentCell.IsModified = false;
		}

		/// <override/>
		public override bool Cut()
		{ 
			if (this.HasFocusControl && this.activeRichTextBox.SelectionLength == 0)
				return false;

			if (CurrentCell.HasCurrentCellAt(RowIndex, ColIndex) && HasControlValue)
				this.activeRichTextBox.Cut();

			SetHeight();
			return true; 
		}

		/// <override/>
		public override bool CanPaste()
		{ 
			bool ok = false;
			foreach(string s in Clipboard.GetDataObject().GetFormats())
			{
				if(s == "Rich Text Format Without Objects")
				{
					ok = true;
					break;
				}
			}
			return ok;
		}

		/// <override/>
		public override bool CanCopy()
		{ 
			return  activeRichTextBox.SelectionLength != 0;
			
		}

		/// <override/>
		public override bool CanCut()
		{ 
			return  activeRichTextBox.SelectionLength != 0;
			
		}

		/// <override/>
		public override bool Copy()
		{ 
			if (this.HasFocusControl && this.activeRichTextBox.SelectionLength == 0)
				return false;
			if (CurrentCell.HasCurrentCellAt(RowIndex, ColIndex) && HasControlValue)
			{
				this.activeRichTextBox.Copy();
			}
			return true; 
		}
	
		/// <override/>
		public override bool Paste()
		{ 
			if (this.activeRichTextBox.ReadOnly)
				return false;

			if (CurrentCell.HasCurrentCellAt(RowIndex, ColIndex) && HasControlValue)
			{
				this.activeRichTextBox.Paste();
				SetHeight();
				return true;
			}
			
			return false;
		}
	
		/// <override/>
		protected override void OnDraw(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style) 
		{
			Rectangle textRectangle = RemoveMargins(clientRectangle, style);

			if (CurrentCell.HasCurrentCellAt(rowIndex, colIndex) && 
				HasControlValue && ShouldDrawFocused(rowIndex, colIndex) && CurrentCell.IsEditing)
			{
				// Position current grid
				activeRichTextBox.Bounds = textRectangle;//clientRectangle;
				activeRichTextBox.BackColor = style.BackColor;
				if(!activeRichTextBox.Focused)
					activeRichTextBox.Focus();

			}
			else
			{
				string rtf;
			
				if (CurrentCell.HasCurrentCellAt(rowIndex, colIndex) && CurrentCell.IsModified && HasControlValue)
					rtf = ControlValue.ToString();
				else
					rtf = style.Text;

				Rectangle clipBounds = GetCellBoundsCore(rowIndex, colIndex, true);
				Color backColor = Color.FromArgb(255, style.Interior.BackColor);
				textRectangle = new Rectangle(textRectangle.X + 1, textRectangle.Y, textRectangle.Width - 3, textRectangle.Height);
				
				//in versions >=  2.0.6.0, use
				//RichTextPaint.DrawRichText(g, staticRichTextBox, rtf, Grid.PrintingMode, Grid.GridBounds, textRectangle, 
				//	clipBounds, backColor, style.WrapText, 100, Grid.IsRightToLeft());
			
				RichTextPaint.DrawRichText(g, staticRichTextBox, rtf, Grid.PrintingMode, Grid.GridBounds, textRectangle, 
					clipBounds, backColor, style.WrapText, 100);
			}
		}

		/// <override/>
		protected override bool OnSaveChanges()
		{
			if (HasControlValue)
			{
				Grid.Model[RowIndex, ColIndex].CellValue = this.activeRichTextBox.Rtf;
				this.Grid.CurrentCell.IsModified = false;
			}
			return true;
		}
		

		/// <override/>
		protected override void OnClick(int rowIndex, int colIndex, MouseEventArgs e)
		{
			if (this.Grid.CurrentCell.HasCurrentCellAt(rowIndex, colIndex))
			{
				// Activate textbox and show caret 
				GridCellLayout layout = GetCellLayout(rowIndex, colIndex, Grid.Model[rowIndex, colIndex]);
				bool clickOnCell = layout.ClientRectangle.Contains(new Point(e.X, e.Y));
			
				bool beginEdit = false;
				if (clickOnCell && (Grid.Model.Options.ActivateCurrentCellBehavior & (GridCellActivateAction.ClickOnCell)) != 0
					|| (Grid.Model.Options.ActivateCurrentCellBehavior & GridCellActivateAction.SetCurrent) != 0)
					beginEdit = true;

				if (beginEdit)
				{
					CurrentCell.BeginEdit();
					if ((Grid.Model.Options.ActivateCurrentCellBehavior & GridCellActivateAction.SelectAll) == 0)
					{
						if (e.Button == MouseButtons.Left 
							&& (Grid.Model.Options.ActivateCurrentCellBehavior & GridCellActivateAction.PositionCaret) != 0)
						{
							Grid.Update();
							Point loc = layout.TextRectangle.Location;
							Point p = new Point(e.X-loc.X, e.Y-loc.Y);
							Syncfusion.Drawing.ActiveXSnapshot.FakeLeftMouseClick(this.activeRichTextBox, p);
						}
						else
						{
							Point loc = layout.TextRectangle.Location;
							Point p = new Point(e.X-loc.X, e.Y-loc.Y);
							ControlMouseDown(this, new MouseEventArgs(e.Button, e.Clicks, p.X, p.Y, 0));
						}
					}
					else
					{
						Point loc = layout.TextRectangle.Location;
						Point p = new Point(e.X-loc.X, e.Y-loc.Y);
						ControlMouseDown(this, new MouseEventArgs(e.Button, e.Clicks, p.X, p.Y, 0));
					}
				}
			}
		
			base.OnClick(rowIndex, colIndex, e);
		}

		protected override void OnKeyPress(KeyPressEventArgs e)
		{

			bool controlKeyDown = (Control.ModifierKeys & Keys.Control) != Keys.None;
			bool menuKeyDown = (Control.ModifierKeys & Keys.Alt) != Keys.None;

			if (!e.Handled && !controlKeyDown && !menuKeyDown)
			{
				if (!Char.IsControl(e.KeyChar)
					&& !IsReadOnly()
					&& SupportsFocusControl)
				{
					if(!CurrentCell.IsEditing)
					{
						GridCellLayout layout = GetCellLayout(CurrentCell.RowIndex,
							CurrentCell.ColIndex, Grid.Model[CurrentCell.RowIndex,
							CurrentCell.ColIndex]);

						this.FixControlParent(activeRichTextBox);

						activeRichTextBox.Size = new Size(layout.ClientRectangle.Width,
							layout.ClientRectangle.Height - 1);
						activeRichTextBox.Location = new Point(layout.ClientRectangle.X,
							layout.ClientRectangle.Y + 1);
						activeRichTextBox.Focus();

						CurrentCell.BeginEdit();
						CurrentCell.HasControlFocus = true;
						this.activeRichTextBox.SelectionStart = 0;
						this.activeRichTextBox.SelectionLength =
							this.activeRichTextBox.Text.Length;

					}

					string s = this.activeRichTextBox.Rtf;
					e.Handled = true;
					int selStart = this.activeRichTextBox.SelectionStart;
					int selLength = this.activeRichTextBox.SelectionLength;
					this.activeRichTextBox.SelectedText = e.KeyChar.ToString();
					string s1 = this.activeRichTextBox.Text;
					if (!this.ValidateString(s1))
					{

						this.activeRichTextBox.Rtf = s;
						this.activeRichTextBox.SelectionStart = selStart;
						this.activeRichTextBox.SelectionLength = selLength;
					}
					else
						CurrentCell.IsModified = true;
				}

			}

			base.OnKeyPress(e);
		}



		/// <override/>
		protected override void OnKeyDown(KeyEventArgs e)
		{
			if(!this.CurrentCell.IsEditing)
				return;

			if (!e.Handled)
			{
				switch (e.KeyCode)
				{
					case Keys.Left:
						this.activeRichTextBox.SelectionStart -=
							(this.activeRichTextBox.SelectionStart > 0) ? 1 : 0;
						this.activeRichTextBox.SelectionLength = 0;
						e.Handled = true;
						break;
					case Keys.Right:
						this.activeRichTextBox.SelectionStart += 1;
						this.activeRichTextBox.SelectionLength = 0;
						e.Handled = true;
						break;
					case Keys.Up:
					{
						int start = this.activeRichTextBox.SelectionStart;
						int lineNo = this.activeRichTextBox.GetLineFromCharIndex(start);
						Point p = this.activeRichTextBox.GetPositionFromCharIndex(start);
						while(lineNo > 0 && start > 0
							&& ( lineNo == this.activeRichTextBox.GetLineFromCharIndex(start)
							|| (lineNo - 1 ==  this.activeRichTextBox.GetLineFromCharIndex(start)
							&& this.activeRichTextBox.GetPositionFromCharIndex(start).X > p.X)
							))
						{
							start -= 1;
						}
						if(lineNo - 2 == this.activeRichTextBox.GetLineFromCharIndex(start))
							start += 1;

						p = this.activeRichTextBox.GetPositionFromCharIndex(start);

						Syncfusion.Drawing.ActiveXSnapshot.FakeLeftMouseClick(this.activeRichTextBox
							,p);
						e.Handled = true;
						break;
					}
					case Keys.Down:
					{
						int start = this.activeRichTextBox.SelectionStart;
						int lineNo = this.activeRichTextBox.GetLineFromCharIndex(start);
						Point p = this.activeRichTextBox.GetPositionFromCharIndex(start);
						while(start < this.activeRichTextBox.TextLength
							&& ( lineNo == this.activeRichTextBox.GetLineFromCharIndex(start)
							|| (lineNo + 1 == this.activeRichTextBox.GetLineFromCharIndex(start)
							&& this.activeRichTextBox.GetPositionFromCharIndex(start).X < p.X)
							))
						{
							start += 1;
						}
						if(lineNo + 2 == this.activeRichTextBox.GetLineFromCharIndex(start))
							start -= 1;

						if(lineNo < this.activeRichTextBox.GetLineFromCharIndex(start))
						{
							p = this.activeRichTextBox.GetPositionFromCharIndex(start);

							Syncfusion.Drawing.ActiveXSnapshot.FakeLeftMouseClick(this.activeRichTextBox
								,p);
						}
						e.Handled = true;
						break;
					}
					case Keys.Back:
					case Keys.Delete:
						if (!CurrentCell.HasControlFocus || !CurrentCell.IsEditing)
						{
							if (Control.ModifierKeys != Keys.Alt)
							{
								GridRangeInfoList rl = Grid.Model.Selections.Ranges;
								if (Control.ModifierKeys == Keys.None
									&& (rl.Count == 0 // no ranges of cells selected
									|| rl.Count == 1 && rl[0] == GridRangeInfo.Cell(this.RowIndex,
									this.ColIndex)))
								{
									if (!IsReadOnly()        // cell is not readonly
										&& OnDeleting()
										&& Grid.RaiseCurrentCellDeleting()       
										// OnDeleteCell	notification returns true
										)
									{
										// Delete text, SetTextBoxText call BeginEdit()
										this.activeRichTextBox.SelectedText = "";
										CurrentCell.IsModified = true;
										CurrentCell.ScrollInView();
									}
									e.Handled = true;
								}
								//								else if (Control.ModifierKeys == Keys.Shift)
								//								{
								//									Grid.Model.CutPaste.Cut();
								//								}
								//								else
								//								{
								//									Grid.Model.Clear(Control.ModifierKeys == Keys.Control);
								//								}
							}
						}
						else
						{
							if (this.activeRichTextBox.SelectionLength > 0)
								this.activeRichTextBox.SelectedText = "";
							else if (this.activeRichTextBox.SelectionStart > 0)
							{
								int offset = e.KeyCode == Keys.Delete ? 0 : 1;

								this.activeRichTextBox.Select(this.activeRichTextBox.SelectionStart -
									offset, 1);
								this.activeRichTextBox.SelectedText = "";
							}
							e.Handled = true;
						}

						break;

					case Keys.Escape:
						if (CurrentCell.IsDroppedDown)
						{

							CurrentCell.CloseDropDown(Syncfusion.Windows.Forms.PopupCloseType.Canceled);
							e.Handled = true;
						}
						else if (CurrentCell.IsModified)
						{
							CurrentCell.RejectChanges();
							CurrentCell.CancelEdit();
							CurrentCell.Refresh();
							CurrentCell.ScrollInView();
							e.Handled = true;
						}
						else
							CurrentCell.CancelEdit();

						if ((Grid.Model.Options.ActivateCurrentCellBehavior &
							Syncfusion.Windows.Forms.Grid.GridCellActivateAction.SetCurrent) != 0)
							CurrentCell.BeginEdit();
						return;

					case Keys.End:
						if (Control.ModifierKeys == Keys.None && !CurrentCell.HasControlFocus)
						{

							if (CurrentCell.BeginEdit())
							{
								// position caret
								CurrentCell.HasControlFocus = true;
								this.activeRichTextBox.SelectionStart =
									this.activeRichTextBox.Text.Length;
								this.activeRichTextBox.SelectionLength = 0;
								CurrentCell.ScrollInView();
								e.Handled = true;
							}
						}
						break;

					case Keys.Home:
						if (Control.ModifierKeys == Keys.None &&
							!CurrentCell.HasControlFocus )
						{
							if (CurrentCell.BeginEdit())
							{
								// position caret
								CurrentCell.HasControlFocus = true;
								this.activeRichTextBox.SelectionStart = 0;
								this.activeRichTextBox.SelectionLength = 0;
								CurrentCell.ScrollInView();
								e.Handled = true;
							}
						}
						break;

					case Keys.Enter:
						if (Control.ModifierKeys == Keys.None && CurrentCell.IsEditing &&
							this.StyleInfo.AllowEnter && this.NotifyCurrentCellChanging())
						{
							this.activeRichTextBox.SelectedText = "\n";
							CurrentCell.ScrollInView();
							e.Handled = true;
						}
						break;
				}
			}

			base.OnKeyDown(e);
		}
		#endregion
	
		public override bool ReplaceText(string find, string replace, int rowIndex, int colIndex, GridFindTextOptions options, bool bSetCell)
		{
			bool wholeCell = (options & GridFindTextOptions.MatchWholeCell) != 0;
			bool matchCase = (options & GridFindTextOptions.MatchCase) != 0;

			GridStyleInfo style = Grid.Model[rowIndex, colIndex];
			string text = style.FormattedText;
			
			AssignRtf(staticRichTextBox, text);
			text = staticRichTextBox.Text;
			CultureInfo culture = style.GetCulture(true);

			string text2 = text;
			if (!matchCase)
			{
				find = find.ToLower(culture);
				text2 = text.ToLower(culture);
			}

			bool found = false;
			if (wholeCell)
			{
				found = find.Equals(text2);
				if (found)
				{
					if (bSetCell)
					{
						CurrentCell.MoveTo(rowIndex, colIndex);
						CurrentCell.BeginEdit();
						AssignRtf(activeRichTextBox, replace);
					}
					else 
					{
						Grid.Model[rowIndex, colIndex].FormattedText = replace;
					}
				}
			}
			else
			{
				int index = text2.IndexOf(find.ToString());
				if (index != -1)
				{
					found = true;
					StringBuilder sb = new StringBuilder();
					if (index > 0)
						sb.Append(text.Substring(0, index));
					sb.Append(replace);
					index += find.Length;
					int index2 = -1;
					do
					{
						text2 = text.Substring(index);
						if (!matchCase)
							text2 = text2.ToLower(culture);
						index2 = text2.IndexOf(find.ToString());
						if (index2 != -1)
						{
							if (index2 > 0)
								sb.Append(text.Substring(index, index2));
							sb.Append(replace);
							index += index2 + find.Length;
						}
					}
					while (index2 != -1);
					if (index < text.Length)
						sb.Append(text.Substring(index));
					if (bSetCell)
					{
						CurrentCell.MoveTo(rowIndex, colIndex);
						CurrentCell.BeginEdit();
						AssignRtf(activeRichTextBox, sb.ToString());
					}
					else 
					{
						Grid.Model[rowIndex, colIndex].FormattedText = sb.ToString();
					}
				}
			}

			return found;
			
			//return base.ReplaceText (find, replace, rowIndex, colIndex, options, bSetCell);
		}
	
		 
	}
	#endregion

}
